#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : diags.py
# Contents: create class probabilities aggregation diagrams
# Author  : Christian Borgelt
# History : 2019.07.21 file created (from some other diags.py)
#           2019.07.22 name generation for images added
#           2019.07.23 probability density type added
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from id2par  import get_args

#-----------------------------------------------------------------------

def header (out, pro, beta, eps):
    xn,yn = len(beta),len(eps)
    out.write('verbatimtex\n')
    out.write('\\documentclass[10pt]{article}\n')
    #out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hdc{\\hbox to0pt{\\phantom{Hgp)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    # out.write('prologues := %d;\n\n' % pro)
    out.write('input axes;\n')
    out.write('input barchart;\n\n')
    out.write('picture WinRate;  '
             +'WinRate = btex \\small win~rate\\hdc etex;\n')
    out.write('picture Beta;     '
             +'Beta    = btex \\small steepness~$\\beta$\\hdc etex;\n')
    out.write('picture Epsilon;  '
             +'Epsilon = btex \\small min.imp.~$\\varepsilon$\\hdc etex;\n')
    out.write('picture Empty;    '
             +'Empty   = btex \\small \hdc etex;\n')
    out.write('numeric expand; expand = 1/3;\n\n')
    out.write('vardef init (expr tlft, ttop, tbot) =\n')
    out.write('  numeric data[][]; numeric n; n := 0;\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.set_units(24mm/%g,' % xn
                                 +' 18mm/%g,' % yn
                                 +' 20mm/100);\n')
    out.write('  barchart.set_ranges((%d,0.84,0.08), ' % xn
                                   +'(%d,0.84,0.08), ' % yn
                                   +'(0,100));\n')
    out.write('  barchart.set_view (142, 36);\n')
    out.write('  barchart.set_light(50, 75);\n')
    out.write('  barchart.mincolor     := red;\n')
    out.write('  barchart.midcolor     := white;\n')
    out.write('  barchart.maxcolor     := 0.8green;\n')
    out.write('  axes.title.size       := 1.0;  % 12pt\n')
    #out.write('  axes.title.skip       := 1.0 *fontsize "ptmr8r";\n')
    out.write('  axes.title.skip       := 3mm;\n')
    out.write('  axes.title.offset     := (2mm,1mm);\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4.2mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4.2mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.alabel.offset  := (0,5mm);\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  barchart.draw_xlabel(0, Beta);\n')
    out.write('  barchart.draw_xticks("%d"' % beta[0])
    for i in range(1,xn): out.write(',"%d"' % beta[i])
    out.write(');\n')
    out.write('  barchart.draw_ylabel(0, Epsilon);\n')
    out.write('  barchart.draw_yticks("%g%%"' % (eps[0]*100))
    for i in range(1,yn): out.write(',"%g%%"' % (eps[i]*100))
    out.write(');\n')
    out.write('  barchart.draw_zlabel(0, WinRate);\n')
    out.write('  barchart.draw_zticks(0,20,40,60,80,100);\n')
    out.write('  barchart.draw_title(-3, -2, tlft, "");\n')
    out.write('  barchart.draw_title(+2, -2, ttop, "");\n')
    out.write('  barchart.draw_title(+2, -2, Empty, tbot);\n')
    #out.write('  barchart.draw_title(+2, -2, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')
    out.write('vardef set_data (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def diag (out, fid, win, cnt, xdist, error, objfn, mode, data):
    out.write('beginfig(%d); %% %s_n%d_x%s_e%s_o%s_m%s\n'
              % (fid, win, cnt, xdist, error, objfn, mode))
    if objfn == '--': objfn = ''
    out.write('  init(btex \\small %s\\hdc etex,\n'  % win)
    out.write('       btex \\small %s\\hdc etex,\n'  % (objfn+mode))
    out.write('       btex \\small %d\\hdc etex);\n' % cnt)
    for d in data:
        d = ','.join(['%.6f' % (x*100) for x in d])
        out.write('  set_data(%s);\n' % d)
    out.write('  barchart.draw_data(data, 0, 3, expand);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    fn_inp = argv[1] if len(argv) > 1 else 'brreg.txt'
    fn_out = argv[2] if len(argv) > 2 else 'diags.mp'
    with open(fn_inp, 'r') as inp:
        data = [l.rstrip('\r\n').split() for l in inp]
    hdr   = data[0]             # read the experimental results
    data  = [[int(d[0])] + d[1:5] + [float(x) for x in d[5:]]
             for d in data[1:]]
    cnts  = sorted(set([d[0] for d in data]))
    xdsts = sorted(set([d[1] for d in data]))
    errs  = sorted(set([d[2] for d in data]))
    modes = sorted(set([(d[3],d[4]) for d in data]))
    eps   = sorted(set([d[5] for d in data]))
    beta  = sorted(set([d[6] for d in data]))
    gamma = sorted(set([d[7] for d in data]))
    with open(fn_out, 'w') as out:
        fid = 0
        header(out, 2, beta, eps)
        for n in cnts:
            for x in xdsts:
                for y in errs:
                    for o,m in modes:
                        dsel = [d for d in data
                                if  d[0] == n and d[1] == x
                                and d[2] == y and d[3] == o
                                and d[4] == m]
                        dwin = [[d[ 9] for e in eps
                                 for d in dsel
                                 if d[5] == e and d[6] == b]
                                for b in beta]
                        diag(out, fid, "win", n, x, y, o, m, dwin)
                        fid += 1
                        dsse = [[d[10] for e in eps
                                 for d in dsel
                                 if d[5] == e and d[6] == b]
                                for b in beta]
                        diag(out, fid, "sse", n, x, y, o, m, dsse)
                        fid += 1
        #                if fid > 2: break
        #            if fid > 2: break
        #        if fid > 2: break
        #    if fid > 2: break
        footer(out)
